<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package AVIF Image Optimizer
 */

namespace Amasty\AvifImageOptimizer\Model\ImageProcessor\ResolutionToolProcessor;

use Amasty\ImageOptimizer\Api\Data\QueueInterface;
use Amasty\ImageOptimizer\Model\Command\CommandProvider;
use Amasty\ImageOptimizer\Model\ImageProcessor\ResolutionToolProcessor\ToolInterface;
use Amasty\ImageOptimizer\Model\Queue\Queue;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem;

class Cavif implements ToolInterface
{
    /**
     * @var Filesystem\Directory\WriteInterface
     */
    private $mediaDirectory;

    /**
     * @var CommandProvider
     */
    private $commandProvider;

    public function __construct(
        Filesystem $filesystem,
        CommandProvider $commandProvider
    ) {
        $this->mediaDirectory = $filesystem->getDirectoryWrite(DirectoryList::MEDIA);
        $this->commandProvider = $commandProvider;
    }

    public function process(QueueInterface $queue, string $newName): void
    {
        $newQueue = clone $queue;
        $newQueue->setFilename($this->mediaDirectory->getRelativePath($newName));
        $avifCommand = $this->commandProvider->get($newQueue->getAvifTool());
        $filename = (string)$newQueue->getFilename();
        $avif = str_replace(
            '.' .  $newQueue->getExtension(),
            '_'. $newQueue->getExtension() . '.avif',
            $filename
        );

        $avifCommand->run($newQueue, $filename, $avif);
    }

    public function getToolName(): string
    {
        return Queue::AVIF_TOOL;
    }
}
